#include "/lib/noise.glsl"

uniform float far;
uniform mat4 gbufferModelView;
uniform mat4 gbufferModelViewInverse;
uniform float frameTimeCounter;

struct SkyColors {
    vec3 ground;
    vec3 top;
};


SkyColors calc_sky_coef() {
    SkyColors FinalSky;
    const vec3 SKY_TOP_NOON = vec3(0.2, 0.36, 0.7);
    const vec3 SKY_TOP_SUNRISE = vec3(0.1, 0.25, 0.5);
    const vec3 SKY_TOP_SUNSET = vec3(0.07, 0.12, 0.15);
    const vec3 SKY_TOP_NIGHT = vec3(0.02, 0.03, 0.05);

    const vec3 SKY_GROUND_NOON = vec3(0.7, 0.8, 1.1);
    const vec3 SKY_GROUND_SUNRISE = vec3(1.0, 0.55, 0.75);
    const vec3 SKY_GROUND_SUNSET = vec3(1.0, 0.4, 0.2);
    const vec3 SKY_GROUND_NIGHT = vec3(0.15, 0.2, 0.3);

    if(worldTime > 23500 || worldTime <= 500) {
        float x = sin(PI*(worldTime+500)/2000.0);
        FinalSky.top = mix(SKY_TOP_NIGHT, SKY_TOP_SUNRISE, x);
        FinalSky.ground = mix(SKY_GROUND_NIGHT, SKY_GROUND_SUNRISE, x);
    }
    else if(worldTime <= 1500) {
        float x = sin(PI*(worldTime-500)/2000.0);
        FinalSky.top = mix(SKY_TOP_SUNRISE, SKY_TOP_NOON, x);
        FinalSky.ground = mix(SKY_GROUND_SUNRISE, SKY_GROUND_NOON, x);
    }
    else if(worldTime <= 11500) {
        FinalSky.top = SKY_TOP_NOON;
        FinalSky.ground = SKY_GROUND_NOON;
    }
    else if(worldTime <= 12500) {
        float x = sin(PI*(worldTime+500.0)/2000.0);
        FinalSky.top = mix(SKY_TOP_NOON, SKY_TOP_SUNSET, x);
        FinalSky.ground = mix(SKY_GROUND_NOON, SKY_GROUND_SUNSET, x);
    }
    else if(worldTime <= 13500) {
        float x = sin(PI*(worldTime-500.0)/2000.0);
        FinalSky.top = mix(SKY_TOP_SUNSET, SKY_TOP_NIGHT, x);
        FinalSky.ground = mix(SKY_GROUND_SUNSET, SKY_GROUND_NIGHT, x);
    }
    else if(worldTime <= 23500) {
        FinalSky.top = SKY_TOP_NIGHT;
        FinalSky.ground = SKY_GROUND_NIGHT;
    }
    FinalSky.top = apply_saturation(FinalSky.top, 1 - rainStrength) * (1 - rainStrength*0.25);
    FinalSky.ground *= (1 - rainStrength*0.25);
    return FinalSky;
} 

float fogify(float x, float w) {
	return w / (x * x + w);
}

vec3 get_clouds(vec3 ViewPos, vec3 CloudPos) {
    float Animation = frameTimeCounter/45+float(worldTime)*PI/5000;
    
	float Noise = fbm(CloudPos.xz+Animation, 5);
    Noise = pow(Noise, CLOUD_AMOUNT - rainStrength*2);

    // Reduce cloud amount further away
    Noise = Noise * (1 - clamp(length(CloudPos.xz/CloudPos.y/24), 0, 1));

    #ifndef DIMENSION_END
    SkyColors SkyCol = calc_sky_coef();
	return dither(Noise * SkyCol.ground, CloudPos.xz);
    #else
    return dither(pow(Noise, 1.33) * vec3(40, 15, 2), CloudPos.xz);
    #endif
}

vec3 calcSkyColor(vec3 pos) {
	float upDot = dot(pos, gbufferModelView[1].xyz)+0.1; //not much, what's up with you?
    SkyColors FinalSky = calc_sky_coef();
	return dither(mix(FinalSky.top, FinalSky.ground, fogify(max(upDot, 0.0), 0.05)), pos.xy);
}
