#include "/lib/fog.glsl"
#include "/global/lighting.fsh"

uniform sampler2D colortex0;
uniform sampler2D colortex2;
uniform sampler2D depthtex0;

uniform mat4 gbufferProjectionInverse;
uniform vec3 fogColor;
uniform vec3 cameraPosition;
// Fog, Clouds

varying vec2 texcoord;
varying vec2 LightmapCoords;

vec3 to_view_pos(vec3 p) {
	vec4 iProjDiag = vec4(gbufferProjectionInverse[0].x, gbufferProjectionInverse[1].y, gbufferProjectionInverse[2].zw);
  	vec3 p3 = p * 2. - 1.;
  	vec4 fragposition = iProjDiag * p3.xyzz + gbufferProjectionInverse[3];
  	return fragposition.xyz / fragposition.w;
}

/* DRAWBUFFERS:0 */

void main() {
	gl_FragData[0] = texture(colortex0, texcoord);

	vec4 Depth = texture(depthtex0, texcoord);

	vec3 ViewPos = to_view_pos(vec3(texcoord, Depth));
	vec3 PlayerPos = (gbufferModelViewInverse * vec4(ViewPos, 1)).xyz;

	#ifdef DIMENSION_OVERWORLD
	vec3 SkyColor = texture(colortex2, texcoord).rgb;
	#elif defined DIMENSION_NETHER
	float LightenFactor = abs(1.7 - get_luminance(fogColor.xyz)); // Makes all biomes roughly equally bright
	LightenFactor *= LightenFactor;
	LightenFactor -= 0.5;
	vec3 SkyColor = apply_saturation(fogColor, 0.7)*LightenFactor;
	#else
	vec3 SkyColor = fogColor*1.5;
	#endif

	float Dist = length(PlayerPos);
	if(Depth.x >= 1) {
		#ifndef DIMENSION_OVERWORLD
		gl_FragData[0].xyz = SkyColor;
		#endif

		#if defined FANCY_CLOUDS && !defined DIMENSION_NETHER
		if(PlayerPos.y > 0) 
		{
			vec3 CloudPos = normalize(PlayerPos); 
			#ifdef DIMENSION_END
			// Adding 5 here makes it look like a black hole
			CloudPos = vec3((CloudPos.xz + 5) * (1 / CloudPos.y), 1).xzy;
			#else
			CloudPos = vec3((CloudPos.xz) * (1 / CloudPos.y), 1).xzy;
			#endif
			gl_FragData[0].xyz += get_clouds(ViewPos, CloudPos);
		}
		#endif

	}
	#ifdef BORDER_FOG
	else
		gl_FragData[0].xyz = get_border_fog(Dist/far, gl_FragData[0].xyz, SkyColor);
	#endif

	gl_FragData[0].xyz = get_lava_fog(Dist, gl_FragData[0].xyz, fogColor);
}
